package CatalystX::HelpText::ViewRole;
use Moose::Role;
use Try::Tiny;
use Moose::Autobox;
use namespace::autoclean;

requires 'expose_methods';

around expose_methods => sub {
    my ($orig, $self, @args) = @_;
    my $m = $self->$orig(@args) || [];
    [ $m->flatten, 'help_text' ];
};

has helptext_model => (
    is => 'ro',
    default => 'Help',
);

sub help_text {
    my ($self, $c, $key) = @_;
    my $model = try { $c->model($self->helptext_model) };
    if (!$model) {
        $c->log->warn(sprintf("Cannot find the '%s' model for %s", $self->helptext_model, ref($self))) if $c->debug;
        return;
    }
    if (!$model->can('get_help_text_for')) {
        $c->log->warn(sprintf("Your '%s' model used by '%s' does not have a get_help_text_for method", $self->helptext_model, ref($self))) if $c->debug;
        return;
    }
    try {
        $model->get_help_text_for($key);
    }
    catch {
        $c->log->warn("Error retrieving help_text key '$key' from model: $_")
            if $c->debug;
        return '';
    };
}

1;

=head1 NAME

CatalystX::HelpText::ViewRole - Role to be applied to Views

=head1 SYNOPSIS

  package MyApp::View::HTML;
  use Moose;

  extends 'Catalyst::View::TT';
  with 'CatalystX::HelpText::ViewRole';

  ... then, in your template code ...

  [% helptext('SomeHelpTopic') %]

=cut

